@@ -10,7 +10,7 @@ use itertools::Itertools;
 use k8s_openapi::apimachinery;
 use k8s_openapi::ByteString;
 use kube::api;
-use kube::{Resource, ResourceExt};
+use kube::ResourceExt;
 use serde::{de, ser};
 
 use super::*;
