@@ -226,61 +226,104 @@ impl super::Kubectl {
         }
     }
 
-    pub async fn update_config_map(
-        &self,
+    /// try to `commit` changes (`update_fn`)
+    /// to object `K`
+    /// of name: `name`
+    /// for at most `max_retries` retries
+    ///
+    /// Note, errors of type `kube::Error::Api` are synthesized in cases of failures
+    pub async fn entry<K>(
+        client: kube::api::Api<K>,
         name: &str,
-        mut update_fn: impl FnMut(BTreeMap<String, String>) -> BTreeMap<String, String>,
-    ) -> kube::Result<corev1::ConfigMap> {
+        max_retries: u8,
+        mut update_fn: impl FnMut(&mut K),
+        default: Option<impl Fn() -> K>,
+    ) -> kube::Result<()>
+    where
+        K: std::fmt::Debug
+            + Clone
+            + serde::de::DeserializeOwned
+            + serde::Serialize
+            + kube::Resource,
+    {
+        /// 1000 minutes maximum backoff times
+        const MAX_BACKOFF_MILLI: u64 = 1_000 * 1_000 * 60;
         const NOT_FOUND: http::StatusCode = http::StatusCode::NOT_FOUND;
         const CONFLICT: http::StatusCode = http::StatusCode::CONFLICT;
-        const INTERNAL_SERVER_ERROR: http::StatusCode = http::StatusCode::INTERNAL_SERVER_ERROR;
-        const RETRY_COUNT: u8 = 16;
+        const UNPROCESSABLE_ENTITY: http::StatusCode = http::StatusCode::UNPROCESSABLE_ENTITY;
 
-        let pp = self.post_params();
-        let configmaps = self.configmaps(None);
+        use kube::api::entry::CommitError;
+        use kube::api::entry::Entry;
+        use kube::error::ErrorResponse;
+        let default = default.as_ref();
 
         let mut last_err = None;
-        for retry in 0..RETRY_COUNT {
-            match configmaps.get(name).await {
-                Ok(mut existing) => {
-                    let data = std::mem::take(&mut existing.data).unwrap_or_default();
-                    let data = update_fn(data);
-                    let cm = if let Some(resource_version) = existing.resource_version() {
-                        corev1::ConfigMap::new(name)
-                            .data(data)
-                            .with_resource_version(resource_version)
-                    } else {
-                        continue;
-                    };
+        for retry_count in 0..max_retries {
+            let entry = match client.entry(name).await? {
+                Entry::Occupied(entry) => entry,
+                Entry::Vacant(entry) => {
+                    let default = default.ok_or_else(|| {
+                        kube::Error::Api(ErrorResponse {
+                            status: NOT_FOUND.to_string(),
+                            code: NOT_FOUND.as_u16(),
+                            message: String::new(),
+                            reason: String::new(),
+                        })
+                    })?;
+                    entry.insert(default())
+                }
+            };
 
-                    match configmaps.replace(name, &pp, &cm).await {
-                        // if we got `CONFLICT` then either uid or resourceVersion do not match
-                        // sleep with a backoff and retry
-                        Err(kube::Error::Api(e)) if e.code == CONFLICT.as_u16() => {
-                            last_err = Some(e);
-                        }
-                        result => return result,
-                    }
+            let e = match entry.and_modify(&mut update_fn).commit().await {
+                Err(CommitError::Save(kube::Error::Api(e))) if e.code == CONFLICT.as_u16() => e,
+                // return for all other case
+                Ok(()) => return Ok(()),
+                Err(CommitError::Save(e)) => return Err(e),
+                Err(ref err @ CommitError::Validate(ref e)) => {
+                    return Err(kube::Error::Api(ErrorResponse {
+                        status: UNPROCESSABLE_ENTITY.to_string(),
+                        code: UNPROCESSABLE_ENTITY.as_u16(),
+                        message: e.to_string(),
+                        reason: err.to_string(),
+                    }))
                 }
-                Err(kube::Error::Api(e)) if e.code == NOT_FOUND.as_u16() => {
-                    let data = update_fn(BTreeMap::new());
-                    let cm = corev1::ConfigMap::new(name).data(data);
-                    return configmaps.create(&pp, &cm).await;
-                }
-                Err(e) => return Err(e),
-            }
-            tokio::time::sleep(Duration::from_millis((1 << retry) * 10)).await
+            };
+            last_err = Some(e);
+
+            let millis = 1u64
+                .checked_shl(retry_count as u32)
+                .and_then(|pow2| pow2.checked_mul(10))
+                .filter(|&millis| millis < MAX_BACKOFF_MILLI)
+                .unwrap_or(MAX_BACKOFF_MILLI);
+
+            tokio::time::sleep(Duration::from_millis(millis)).await
         }
 
         let e = last_err.unwrap_or_else(|| kube::error::ErrorResponse {
-            code: INTERNAL_SERVER_ERROR.as_u16(),
-            status: INTERNAL_SERVER_ERROR.as_str().to_string(),
+            code: CONFLICT.as_u16(),
+            status: CONFLICT.as_str().to_string(),
             message: String::new(),
             reason: String::new(),
         });
         Err(kube::Error::Api(e))
     }
 
+    pub async fn update_config_map(
+        &self,
+        name: &str,
+        mut update_fn: impl FnMut(&mut BTreeMap<String, String>),
+    ) -> kube::Result<()> {
+        const MAX_RETRY: u8 = 16;
+        Self::entry(
+            self.configmaps(None),
+            name,
+            MAX_RETRY,
+            |cm| update_fn(cm.data.get_or_insert_with(BTreeMap::default)),
+            Some(|| corev1::ConfigMap::new(name)),
+        )
+        .await
+    }
+
     pub async fn list_daemonsets<'a>(
         &self,
         namespace: impl Into<Option<&'a str>>,
